"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Docker = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const dockerode_1 = (0, tslib_1.__importDefault)(require("dockerode"));
const archiver_1 = require("archiver");
const temp_1 = (0, tslib_1.__importDefault)(require("temp"));
const fs = (0, tslib_1.__importStar)(require("node:fs"));
const node_path_1 = require("node:path");
const node_stream_1 = require("node:stream");
class Docker {
    constructor(options) {
        this.docker = new dockerode_1.default(options);
    }
    async ping() {
        try {
            await this.docker.ping();
            return true;
        }
        catch (_a) {
            core_1.CliUx.ux.error('Could not access docker');
            return false;
        }
    }
    async pullImage(image) {
        return new Promise((resolve, reject) => {
            function onFinished(finishedError) {
                if (finishedError) {
                    return reject(finishedError);
                }
                return resolve(true);
            }
            this.docker.pull(image, (err, stream) => {
                if (err) {
                    return reject(err);
                }
                this.docker.modem.followProgress(stream, onFinished);
            });
        }).catch(error => {
            core_1.CliUx.ux.error(`Could not pull image: ${error.message}`);
        });
    }
    async runImage(image, createOptions, outputStream = process.stdout) {
        const stdStream = new node_stream_1.PassThrough();
        const errStream = new node_stream_1.PassThrough();
        stdStream.on('data', chunk => {
            outputStream.write(chunk);
        });
        let success = true;
        errStream.on('data', chunk => {
            core_1.CliUx.ux.error(chunk === null || chunk === void 0 ? void 0 : chunk.toString(), { exit: false });
            success = false;
        });
        Object.assign(createOptions, { Tty: false });
        await this.docker.run(image, [], [stdStream, errStream], createOptions)
            .catch(error => {
            core_1.CliUx.ux.error(`Could not build image: ${error.message}`);
            success = false;
        })
            .then(function (data) {
            if (data[0].StatusCode !== 0) {
                success = false;
            }
        });
        return success;
    }
    async deleteContainer(name) {
        const container = await this.docker.getContainer(name);
        await container.remove({ force: true });
    }
    async buildImage(context, dockerfile, integrationName, buildArgs, nocache = false) {
        const tempDir = await temp_1.default.mkdir();
        const tarPath = tempDir + '/context.tar';
        const output = fs.createWriteStream(tarPath);
        const archive = (0, archiver_1.create)('tar');
        archive.pipe(output);
        archive.glob('**/*.*', {
            cwd: context,
            skip: [
                'venv',
                'output',
                'dist',
                '.idea',
                'data',
                '**/__pycache__',
                'test_output',
                '.coverage',
            ],
            ignore: [
                '**/*.swimbundle',
            ],
        });
        const dockerfilePath = (0, node_path_1.resolve)((0, node_path_1.join)(__dirname, dockerfile));
        archive.append(fs.createReadStream(`${dockerfilePath}.Dockerfile`), { name: 'Dockerfile' });
        await archive.finalize();
        const tag = `swsdk:${integrationName}`;
        const options = {
            t: tag,
            forcerm: true,
            rm: true,
            buildargs: buildArgs,
            nocache: nocache,
        };
        const stream = await this.docker.buildImage(tarPath, options);
        try {
            const progress = await new Promise((resolve, reject) => {
                this.docker.modem.followProgress(stream, (error, response) => error ? reject(error) : resolve(response));
            });
            const errors = progress === null || progress === void 0 ? void 0 : progress.filter((x) => x.error);
            if (errors) {
                for (const error of errors) {
                    core_1.CliUx.ux.error(error.error);
                }
            }
        }
        catch (error) {
            core_1.CliUx.ux.error(error);
        }
        return tag;
    }
}
exports.Docker = Docker;
//# sourceMappingURL=docker.js.map